pub struct ArbitraryRegexpPrefix { /* private fields */ }Expand description
Indicates that a regular expression has a prefix that can be arbitrarily long and matches any sequence of bytes.
Example
error[E045]: arbitrary regular expression prefix
--> line:3:11
|
3 | $a = /.*foo/s
| ^^ this prefix can be arbitrarily long and matches all bytes
|Regular expressions with such prefixes are problematic because YARA will report a match at every file offset from the start of the file up to where the rest of the pattern matches. In most cases, this prefix can be removed without affecting the rule’s semantics.
Implementations
Sourceimpl ArbitraryRegexpPrefix
impl ArbitraryRegexpPrefix
Sourcepub const fn code() -> &'static str
pub const fn code() -> &'static str
Returns a unique code identifying the type of error/warning.
Error codes have the form “Eddd”, where “ddd” is an error number (examples: “E001”, “E020”). Warnings have more descriptive codes, like: “slow_pattern”, “unsatisfiable_expr”, etc.
Sourcepub fn labels(&self) -> impl Iterator<Item = Label<'_>>
pub fn labels(&self) -> impl Iterator<Item = Label<'_>>
Returns the labels associated to this error/warning.
Returns the footers associated to this error/warning.