pub struct BooleanIntegerComparison { /* private fields */ }Expand description
Comparison between boolean and integer.
This warning indicates that some expression is a comparison between boolean and integer values.
Example
warning[bool_int_comparison]: comparison between boolean and integer
--> line:4:13
|
4 | condition: test_proto2.array_bool[0] == 1
| ------------------------------ this comparison can be replaced with: `test_proto2.array_bool[0]`
|Implementations
Sourceimpl BooleanIntegerComparison
impl BooleanIntegerComparison
Sourcepub const fn code() -> &'static str
pub const fn code() -> &'static str
Returns a unique code identifying the type of error/warning.
Error codes have the form “Eddd”, where “ddd” is an error number (examples: “E001”, “E020”). Warnings have more descriptive codes, like: “slow_pattern”, “unsatisfiable_expr”, etc.
Sourcepub fn labels(&self) -> impl Iterator<Item = Label<'_>>
pub fn labels(&self) -> impl Iterator<Item = Label<'_>>
Returns the labels associated to this error/warning.
Returns the footers associated to this error/warning.