pub struct SlowPattern { /* private fields */ }Expand description
Some pattern may be potentially slow.
This warning indicates that a pattern may be very slow to match, and can
degrade rule’s the performance. In most cases this is caused by patterns
that doesn’t contain any large fixed sub-pattern that be used for speeding
up the scan. For example, {00 [1-10] 01} is very slow because the only
fixed sub-patterns (00 and 01) are only one byte long.
Example
warning[slow_pattern]: slow pattern
--> line:3:5
|
3 | $a = {00 [1-10] 01}
| ------------------ this pattern may slow down the scan
|Implementations
Sourceimpl SlowPattern
impl SlowPattern
Sourcepub const fn code() -> &'static str
pub const fn code() -> &'static str
Returns a unique code identifying the type of error/warning.
Error codes have the form “Eddd”, where “ddd” is an error number (examples: “E001”, “E020”). Warnings have more descriptive codes, like: “slow_pattern”, “unsatisfiable_expr”, etc.
Sourcepub fn labels(&self) -> impl Iterator<Item = Label<'_>>
pub fn labels(&self) -> impl Iterator<Item = Label<'_>>
Returns the labels associated to this error/warning.
Returns the footers associated to this error/warning.